package cz.fi.muni.pa165.dao;

import cz.fi.muni.pa165.entity.BaseEntity;
import org.springframework.data.repository.NoRepositoryBean;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Root;
import java.util.List;
import java.util.UUID;

/**
 * @author Dominik Kozubík
 */
@NoRepositoryBean
public class BaseDaoImpl<Entity extends BaseEntity> implements BaseDao<Entity> {

    final Class<Entity> entityClass;

    @PersistenceContext
    protected EntityManager em;

    public BaseDaoImpl(Class<Entity> entityClass) {
        this.entityClass = entityClass;
    }

    @Override
    public void create(Entity entity) {
        em.persist(entity);
    }

    @Override
    public List<Entity> findAll() {

        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<Entity> cq = cb.createQuery(entityClass);
        Root<Entity> rootEntry = cq.from(entityClass);
        CriteriaQuery<Entity> all = cq.select(rootEntry);
        TypedQuery<Entity> allQuery = em.createQuery(all);
        return allQuery.getResultList();
    }

    @Override
    public Entity findById(UUID id) {
        return em.find(entityClass, id);
    }

    @Override
    public void remove(Entity entity) {
        em.remove(entity);
    }
}
